/// \file mil1553ud_library.h
/// \brief библиотка взаимодействия
/// заголовки функций
#ifndef MIL1553UD_LIBRARY_H
#define MIL1553UD_LIBRARY_H

#include "mil1553ud_cmd.h"
#include "mil1553ud_regs.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>  /* ioctl */
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <sys/param.h>

#ifdef __cplusplus
extern "C" {
#endif

/// \brief невалидное значение, свидетельствующее об ошибке
#define LIB_BAD_VALUE       0xFFFFFFFF
/// \brief значение отсутсвия ошибки
#define LIB_GOOD            0

/// \remark ОБЩИЕ СЕРВИСНЫЕ ФУНКЦИИ

///
/// \brief mil1553ud_getNumberChannel получить номер канала из символьного утсройства канала
/// \param fd дескриптор файла
/// \return номер канала
///
unsigned int mil1553ud_getNumberChannel(int fd);

///
/// \brief макрос генериует имя регистра в соответствии с номером канала
/// \param numCh номер канала
/// \param RegName имя регистра без номера канала
///
/// \code
/// unsigned int numCh = mil1553ud_getNumberChannel(fd);
/// unsigned int regAddress = MIL_REG(numCh, CTRL_REG_PCI_CH);
///
#define MIL_REG(numCh, RegName) \
    ( (numCh == CH_NUM_1) ? RegName##1 : ( (numCh == CH_NUM_2) ? RegName##2 : ( (numCh == CH_NUM_3) ? RegName##3 : RegName##4) ) )

/// \remark РАБОТА С КОЛЬЦЕВЫМ БУФЕРОМ ДМА

///
/// \brief mil1553ud_getCountDmaBlocksReadyRead получить количество блоков ДМА, готовых для чтения
/// \param fd дескриптор файла
/// \return кол-во блоков ДМА, готовых к чтению
///
unsigned int mil1553ud_getCountDmaBlocksReadyRead(int fd, unsigned int* codeError);

///
/// \brief mil1553ud_newDmaReadBlock создать контейнер блоков ДМА
/// выделяет под контейнер память
/// \param countDmaBlocks размер контейнера в блоках (штуках)
/// \return указатель на контейнер
///
DMA_READ_BLOCK* mil1553ud_newDmaReadBlock(unsigned int countDmaBlocks);

///
/// \brief mil1553ud_freeDmaReadBlock освобождает контейнер блоков ДМА
/// \param block указатель на контейнер
///
void mil1553ud_freeDmaReadBlock(DMA_READ_BLOCK* block);

///
/// \brief mil1553ud_readAllDmaBlocksReadyRead считывает все блоки ДМА, готовые к чтению, из канала
/// выделяет под контейнер память
/// \param fd дескриптор файла
/// \return указатель на контейнер
///
DMA_READ_BLOCK* mil1553ud_readAllDmaBlocksReadyRead(int fd, unsigned int *cntBlocks);

/// \remark РАБОТА С КОЛЬЦЕВЫМ БУФЕРОМ ПРЕРЫВАНИЙ

///
/// \brief mil1553ud_getCountInterruptBlocksReadyRead получить количество блоков прерываний, готовых для чтения
/// \param fd дескриптор файла
/// \return кол-во блоков, готовых к чтению
///
unsigned int mil1553ud_getCountInterruptBlocksReadyRead(int fd);

///
/// \brief mil1553ud_newInterruptBlock создать контейнер блоков прерываний
/// \param countBlocks размер контейнера в блоках (штуках)
/// \return указатель на контейнер
///
INTERRUPT_BLOCK_BUFFER* mil1553ud_newInterruptBlock(unsigned int countBlocks);

///
/// \brief mil1553ud_freeInterruptBlock освобождает контейнер блоков блоков прерываний
/// \param block указатель на контейнер
///
void mil1553ud_freeInterruptBlock(INTERRUPT_BLOCK_BUFFER* block);

///
/// \brief mil1553ud_readAllInterruptBlocksReadyRead считывает все блоки прерываний, готовые к чтению, из канала
/// \param fd дескриптор файла
/// \return указатель на контейнер
///
INTERRUPT_BLOCK_BUFFER* mil1553ud_readAllInterruptBlocksReadyRead(int fd);

/// \remark РЕЖИМ КШ

/// \brief действие - останов программы
#define BC_PROGRAM_STOP     0x0u
/// \brief действие - старт программ
#define BC_PROGRAM_START    ~BC_PROGRAM_STOP

///
/// \brief mil1553ud_bcProgram управление микропрограммой в режиме КШ
/// старт/стоп BCSTRT
/// \param fd дескриптор файла
/// \param action действие
///
unsigned int mil1553ud_bcProgram(int fd, unsigned int action);

///
/// \brief mil1553ud_bcSetStartInstructionAddress установить стартовый адрес интсрукции микропрограммы КШ
/// \param fd дескриптор файла
/// \param startNumber стартовый номер (адрес)
///
unsigned int mil1553ud_bcSetStartInstructionAddress(int fd, unsigned int startNumber);

/// \remark РЕЖИМ КШ - РАБОТА С МИКРОПРОГРАММОЙ КШ

///
/// \brief mil1553ud_newBcRamBlock создать контейнер для микропрограммы КШ
/// \param type тип - инструкции, операции, данные
/// MIL_BC_RAM_TYPE_INSTRUCTION, MIL_BC_RAM_TYPE_OPERATION, MIL_BC_RAM_TYPE_DATA
/// \param sizeInDwords размер контейнера в 32-разядных (двойных) словах
/// \param shiftInDwords смещение в памяти КШ в 32-разядных (двойных) словах
/// \return указатель на контейнер для микропрограммы КШ
///
BC_RAM_BLOCK* mil1553ud_newBcRamBlock(unsigned int type, unsigned int sizeInDwords, unsigned int shiftInDwords);

///
/// \brief mil1553ud_freeBcRamBlock освободить контейнер для микропрограммы КШ
/// \param block указатель на контейнер для микропрограммы КШ
///
void mil1553ud_freeBcRamBlock(BC_RAM_BLOCK* block);

/// \remark РЕЖИМ КШ - РАБОТА С МИКРОПРОГРАММОЙ КШ - ИНСТРУКЦИИ, ОПЕРАЦИИ

#define XEQ         0x1
#define XQF         0x15
#define XQG         0x16
#define XFG         0x1E
#define JMP         0x2
#define CAL         0x3
#define RTN         0x4
#define IRQ         0x6
#define HLT         0x7
#define DLY         0x8
#define WFT         0x9
#define CFT         0xA
#define CMT         0xB
#define LTT         0xD
#define LTH         0x18
#define LFT         0xE
#define SFT         0xF
#define WMP         0x1A
#define WMI         0x1C
#define DSZ         0x1D
#define FLG         0xC

#define IDENT       0xA

#define ALWAYS      0xF

///
/// \brief формирование инструкции (32-разрядное слово)
/// \param instruction инструкция
/// \param condition условие
/// \param parameter параметр
///
/// \code
/// unsigned int dword = INSTRUCTION(XEQ, ALWAYS, 0);
///
#define INSTRUCTION(instruction, condition, parameter)   \
    (   \
    ( 0 << 31 ) | ( (instruction & 0x1F) << 26 ) | ( IDENT << 21 ) | ( (condition & 0x1F) << 16) | (parameter & 0xFFFF) \
    )

///
/// \brief нечётная часть операции (32-разрядное слово)
/// \param command команда КШ
/// \param timeout время до начала следующей операции
///
#define OPERATION_ODD_PART(command, timeout)    \
    (   \
    ( (command & 0xFFFF) << 16 ) | (timeout & 0xFFFF)  \
    )

///
/// \brief чётная часть операции (32-разрядное слово)
/// \param cw1 КС1
/// \param parameter параметр
///
#define OPERATION_EVEN_PART(cw1, parameter) \
    (   \
    ( (cw1 & 0xFFFF) << 16 ) | (parameter & 0xFFFF)  \
    )

///
/// \brief командное слово (ГОСТ 52070)
/// \param adrOu адрес ОУ
/// \param k разряд "приём/передача"
/// 0 - ОУ должен принимать СД
/// 1 - ОУ должен передавать СД
/// \param suba_mode подадрес или режим управления
/// \param cntDw_code число СД или код команды
///
#define COMMAND_WORD(adrOu, k, suba_mode, cntDw_code)  \
    (   \
    ( (adrOu & 0x1F) << 11 ) | ( (k & 0x1) << 10) | ( (suba_mode & 0x1F) << 5) | ( (cntDw_code & 0x1F) << 0)   \
    )

/// \remark РАСШИФРОВКА БЛОКОВ ДМА

/// \remark РАСШИФРОВКА БЛОКОВ ДМА - СЕРВИСНЫЕ КОНСТАНТЫ

/// \brief тип блока ДМА - КШ
#define MIL1553UD_BC_DMA_BLOCK      0
/// \brief тип блока ДМА - ОУ
#define MIL1553UD_TD_DMA_BLOCK      1

/// \brief значение FALSE (ложь)
#define MIL1553UD_FALSE             0
/// \brief значение TRUE (истина)
#define MIL1553UD_TRUE              ~(MIL1553UD_FALSE)

/// \brief размер двойного слова в байтах
#define DWORD_SIZE  4
/// \brief размер блока ДМА
#define MIL_DMA_BLOCK_SIZE 128

/// \remark РАСШИФРОВКА БЛОКОВ ДМА - СЕРВИСНЫЕ МАКРОСЫ

/// \brief получить указатель на блок ДМА
#define GET_PTR_DMA_BLOCK(data, numberBlock) \
    data + MIL_DMA_BLOCK_SIZE * numberBlock

/// \brief проверка на допустимость
#define ASSERT_DMA_CONTAINER(ptrContainerDma, numberBlock) \
    (ptrContainerDma == NULL) ? MIL1553UD_FALSE : \
        (numberBlock > ptrContainerDma->countBlocks) ? MIL1553UD_FALSE : MIL1553UD_TRUE

/// \remark РАСШИФРОВКА БЛОКОВ ДМА - ФУНКЦИИ

///
/// \brief mil1553ud_dmaBlock_getTypeTransaction получить тип транзакции из блока ДМА
/// тип транзакции соттветствует значениям 1-10 (см. ГОСТ 52070)
/// \param dmaBlock тип блока ДМА
/// \param dmaBlock указатель на блок ДМА
/// \return тип транзакции
///
unsigned int mil1553ud_dmaBlock_getTypeTransaction(unsigned int typeDmaBlock, unsigned char* dmaBlock);

///
/// \brief mil1553ud_dmaBlock_isSuccessTransaction проверить успешность транзакции
/// \param typeDmaBlock тип блока ДМА
/// \param dmaBlock указатель на блок ДМА
/// \return признак успешности транзакции
///
unsigned int mil1553ud_dmaBlock_isSuccessTransaction(unsigned int typeDmaBlock, unsigned char* dmaBlock);

///
/// \brief mil1553ud_dmaBlock_getFreeTimer получить значение freetimer блока ДМА
/// \param typeDmaBlock тип блока ДМА
/// \param dmaBlock указатель на блок ДМА
/// \return значение freetimer
///
unsigned int mil1553ud_dmaBlock_getFreeTimer(unsigned int typeDmaBlock, unsigned char* dmaBlock);

///
/// \brief mil1553ud_dmaBlock_getCountServiceWords16 получить количество служебных 16-разрядных слов
/// (включая dword 1-5 и служебные слова МКИО)
/// \param typeDmaBlock тип блока ДМА
/// \param dmaBlock указатель на блок ДМА
/// \return количество служебных 16-разрядных слов
///
unsigned int mil1553ud_dmaBlock_getCountServiceWords16(unsigned int typeDmaBlock, unsigned char* dmaBlock);

///
/// \brief mil1553ud_dmaBlock_getCountDataWords16 получить количество 16-разрядных слов данных
/// \param typeDmaBlock тип блока ДМА
/// \param dmaBlock указатель на блок ДМА
/// \return количество 16-разрядных слов данных
///
unsigned int mil1553ud_dmaBlock_getCountDataWords16(unsigned int typeDmaBlock, unsigned char* dmaBlock);

/// \brief Шина "А"
#define MIL1553_BUS_A       0x0
/// \brief Шина "Б"
#define MIL1553_BUS_B       ~MIL1553_BUS_A

///
/// \brief mil1553ud_dmaBlock_getActiveBus получить активную шину
/// (по которой была транзакция)
/// \param typeDmaBlock тип блока ДМА
/// \param dmaBlock указатель на блок ДМА
/// \return шина А или Б
/// MIL1553_BUS_A или MIL1553_BUS_B
///
unsigned int mil1553ud_dmaBlock_getActiveBus(unsigned int typeDmaBlock, unsigned char* dmaBlock);

///
/// \brief mil1553ud_dmaBlock_getPtrDataPart получить указатель на часть с данными блока ДМА
/// \param dmaBlock указатель на блок ДМА
/// \return указатель на часть с данными блока ДМА
///
unsigned char* mil1553ud_dmaBlock_getPtrDataPart(unsigned char* dmaBlock);

///
/// \brief mil1553ud_dmaBlock_tdModeSubaddress получить поддарес из блока ДМА в режиме ОУ
/// \param dmaBlock указатель на блок ДМА
/// \return поддарес
///
unsigned int mil1553ud_dmaBlock_tdModeSubaddress(unsigned char* dmaBlock);

#ifdef __cplusplus
}
#endif

#endif // MIL1553UD_LIBRARY_H
